#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

#define MAX_LIGHTS 128

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

const float levels = 4.0;

uniform int pointLCnt;
uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} 
pointL[MAX_LIGHTS];

vec3 phong(vec3 n, vec3 direction, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(direction);
	direction = normalize(direction);
	float att = 1.0;	
	if(attenuate){ 
		att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	}
	vec3 ref = reflect(-direction, n);
	float brightness = max(0, dot(n, direction));
	float level = floor(brightness * levels);
	brightness = level / levels;
	return (diffuseF * diffuseC * brightness + specularF * specularC * pow(max(0, dot(ref, v)), alpha)) * att; 
	}

void main() {	

	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;

	//----------------------CEL SHADING----------------------//
    //------------------------COLOR--------------------------//
    
    float r;
    float g;
    float b;
    
    float Cmax;
    float Cmin;
    float delta;
    
    float H;
    float S;
    float V;
	
	const float levelsV = 4.0;
    
    r = texColor.r;
    g = texColor.g;
    b = texColor.b;
    
    Cmax = max(max(r,g),b);
	
    Cmin = min(min(r,g),b);
	
    delta = Cmax - Cmin;
    
    //RGB to HSV
    
    if(delta == 0){
    	H = 0;
    }
    if(Cmax == r){
    	H = ((g-b)/delta);
    }else if(Cmax == g){
    	H = ((b-r)/delta) + 2;
    
    }else if(Cmax == b){
    	H = ((r-g)/delta) + 4;
    }
    
    if(Cmax == 0){
    	S = 0;
    }else{
    	S = delta/Cmax;
    }
    
    V = Cmax;
    
    //STEPPING
	
	float levelV = floor(V * levelsV);
	V = levelV / levelsV;
	
    //HSV to RGB
    
	if(H < 0){
		H = H - 6 * floor(H/6) + 6;
	}else{
		H = H - 6 * floor(H/6);
	}
	
    float rH;
    float gH;
    float bH;
    
    float alpha = V * (1-S);
	float beta = V * (1-(H-floor(H))*S);
    float gamma = V * (1-(1-(H - floor(H))) * S);
	
    
    	if(0.0 <= H && H < 1.0){
    		 rH = V;
    		 gH = gamma;
    		 bH = alpha;
    	}else if(1.0 <= H && H < 2.0){
    		 rH = beta;
    		 gH = V;
    		 bH = alpha;
    	}else if(2.0 <= H && H < 3.0){
    		 rH = alpha;
    		 gH = V;
    		 bH = gamma;
    	}else if(3.0 <= H && H < 4.0){
    		 rH = alpha;
    		 gH = beta;
    		 bH = V;
    	}else if(4.0 <= H && H < 5.0){
    		 rH = gamma;
    		 gH = alpha;
    		 bH = V;
    	}else if(5.0 <= H && H < 6.0){
    		 rH = V;
    		 gH = alpha;
    		 bH = beta;
    	}
    
    	r = rH;
    	g = gH;
    	b = bH;
    
    	texColor.r = r;
    	texColor.g = g;
    	texColor.b = b;
	

	color = vec4(texColor * materialCoefficients.x, 1); // ambient
			
		
	// add directional light contribution
	color.rgb += phong(n, dirL.direction - vert.position_world, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
		


    for(int i = 0; i < pointLCnt; i++) 
	{
		// add point light contribution
		color.rgb += phong(n, pointL[i].position - vert.position_world, v, pointL[i].color * texColor, materialCoefficients.y, pointL[i].color, materialCoefficients.z, specularAlpha, true, pointL[i].attenuation);
    }
	


}

